"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useScrollSpy = void 0;
const react_1 = require("react");
const throttle_1 = require("../utils/throttle");
/**
 * @param ScrollSpyParams Optional params
 * @param ScrollSpyParams.offsetPx Distance from Y coordinate of the base (px)
 * @param ScrollSpyParams.throttleMs Interval of update processing (ms)
 * @returns [ activeEntry, actions ] - Two return values are returned as an array
 * @returns activeEntry - DOM element activated based on scroll position (only one)
 * @returns actions - Actions are used to control registered DOM elements
 */
const useScrollSpy = ({ offsetPx = 0, throttleMs = 100, } = {}) => {
    const [activeEntry, setActiveEntry] = (0, react_1.useState)(null);
    const [entries, setEntries] = (0, react_1.useState)({});
    const setScrollSpyEntry = (key, entry) => {
        setEntries((prevState) => ({
            ...prevState,
            [key]: entry,
        }));
    };
    const deleteScrollSpyEntry = (key) => {
        const deletedEntries = { ...entries };
        delete deletedEntries[key];
        setEntries(deletedEntries);
    };
    const resetScrollSpyEntries = () => {
        setEntries({});
    };
    const updateActiveEntry = () => {
        const entryKeys = Object.keys(entries);
        const activeKey = entryKeys.reduce((prevActiveKey, key) => {
            const entryElement = entries[key].current;
            if (!entryElement) {
                return prevActiveKey;
            }
            const entryDomRect = entryElement.getBoundingClientRect();
            const isPassed = entryDomRect.top + offsetPx < 0;
            if (isPassed) {
                return prevActiveKey;
            }
            const prevEntryElement = entries[prevActiveKey].current;
            if (!prevEntryElement) {
                return key;
            }
            const prevEntryDomRect = prevEntryElement.getBoundingClientRect();
            const isHigherThanPrev = entryDomRect.top >= prevEntryDomRect.top;
            return isHigherThanPrev ? key : prevActiveKey;
        });
        if (!!activeKey) {
            setActiveEntry(null);
            return;
        }
        setActiveEntry({
            key: activeKey,
            value: entries[activeKey],
        });
    };
    (0, react_1.useEffect)(() => {
        const InvokeUpdate = (0, throttle_1.throttle)(updateActiveEntry, throttleMs);
        window.addEventListener("scroll", InvokeUpdate);
        return () => {
            window.removeEventListener("scroll", InvokeUpdate);
        };
    }, [entries]);
    const actions = {
        set: setScrollSpyEntry,
        delete: deleteScrollSpyEntry,
        reset: resetScrollSpyEntries,
    };
    return [activeEntry, actions];
};
exports.useScrollSpy = useScrollSpy;
//# sourceMappingURL=useScrollSpy.js.map